@@ -2,8 +2,9 @@
  *   Go buy a copy.
  *
  * Copyright (C) 2005 Brian Wellington
- * Copyright (C) 2005,2006 Roland Clobus
+ * Copyright (C) 2005,2011 Roland Clobus
  * Copyright (C) 2005,2006 Bas Wijnen
+ * Copyright (C) 2011-2013 Micah Bunting <Amnykon@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,15 +40,20 @@
 #include "scrollable-text-view.h"
 #include "guimap.h"
 #include "theme.h"
+#include "map-icons.h"
 #include "colors.h"
 #include "common_gtk.h"
 #include "cards.h"
+#include "network.h"
 
 #define MAINICON_FILE "pioneers-editor.png"
 
 #define MAP_WIDTH 550		/* default map width */
 #define MAP_HEIGHT 400		/* default map height */
 
+#define BUTTON_HEIGHT 24	/* height of the toolbar buttons */
+#define TERRAIN_BUTTON_WIDTH 27	/* width of the terrain toolbar buttons */
+
 static GtkWidget *toplevel;
 static gchar *default_game;
 static gchar *window_title;
@@ -85,6 +91,24 @@ typedef enum {
 static GuiMap *gmap;
 static Hex *current_hex;
 
+typedef enum {
+	TERRAIN_TOOLBAR_BUTTON_TYPE,
+	CHIT_TOOLBAR_BUTTON_TYPE,
+	PORT_TOOLBAR_BUTTON_TYPE,
+	NO_TOOLBAR_BUTTON_TYPE
+} ToolbarButtonType;
+
+typedef struct {
+	ToolbarButtonType type;
+	union {
+		Terrain terrain;
+		gint chit;
+		Resource port;
+	} element;
+} ToolbarButtonData;
+
+static ToolbarButtonData selected_toolbar_button;
+
 static const gchar *terrain_names[] = {
 	/* Use an unique shortcut key for each resource */
 	N_("_Hill"),
@@ -139,6 +163,7 @@ static const gchar *port_direction_names[] = {
 };
 
 static void check_vp_cb(GObject * caller, gpointer main_window);
+static void set_window_title(const gchar * title);
 
 static void error_dialog(const char *fmt, ...)
 {
@@ -216,6 +241,326 @@ static gboolean terrain_has_chit(Terrain terrain)
 	return FALSE;
 }
 
+/** Changes the terrain of a hex to terrain.
+ * @param hex The hex to change the terrain of.
+ * @param terrain The terrain to change the hex terrain to.
+ */
+static void change_terrain(Hex * hex, Terrain terrain)
+{
+	Hex *adjacent;
+	gint i;
+
+	if (terrain == hex->terrain)
+		return;
+
+	hex->terrain = terrain;
+	if (terrain_has_chit(terrain)) {
+		if (hex->roll == 0)
+			hex->roll = 2;
+	} else
+		hex->roll = 0;
+
+	if (terrain != SEA_TERRAIN)
+		hex->resource = NO_RESOURCE;
+
+	/* If terrain is not land,
+	 * remove all ports on adjacent hexes that point to hex */
+	if (terrain == SEA_TERRAIN || terrain == LAST_TERRAIN) {
+		for (i = 0; i < 6; i++) {
+			adjacent = hex_in_direction(hex, i);
+			if (adjacent != NULL
+			    && adjacent->resource != NO_RESOURCE
+			    && adjacent->facing == (i + 3) % 6) {
+				adjacent->resource = NO_RESOURCE;
+				adjacent->facing = 0;
+				guimap_draw_hex(gmap, adjacent);
+			}
+		}
+	}
+	guimap_draw_hex(gmap, hex);
+	return;
+}
+
+/** Draws the chit in the exposed area.
+ * @param widget The widget to draw on
+ * @param cr The cairo context
+ * @param chip_number The chit number of the hex.
+ * @return TRUE if event is handled.
+ */
+static gboolean draw_chit_cb(GtkWidget * widget, cairo_t * cr,
+			     gpointer chip_number)
+{
+	PangoLayout *layout;
+
+	layout = gtk_widget_create_pango_layout(widget, "");
+	draw_dice_roll(layout, cr,
+		       12, 10, 15, GPOINTER_TO_INT(chip_number),
+		       GOLD_TERRAIN, FALSE);
+	g_object_unref(layout);
+	return TRUE;
+}
+
+/** Draws the port in the exposed area.
+ * @param widget The widget to draw on
+ * @param cr The cairo context
+ * @param port_type The type of the port.
+ * @return TRUE if event is handled.
+ */
+static gboolean draw_port_cb(GtkWidget * widget, cairo_t * cr,
+			     gpointer port_type)
+{
+	PangoLayout *layout;
+	MapTheme *theme;
+	gint width;
+	gint height;
+
+	layout = gtk_widget_create_pango_layout(widget, "");
+
+	theme = theme_get_current();
+	width = gdk_pixbuf_get_width(theme->terrain_tiles[SEA_TERRAIN]);
+	height = gdk_pixbuf_get_height(theme->terrain_tiles[SEA_TERRAIN]);
+	gdk_cairo_set_source_pixbuf(cr, theme->terrain_tiles[SEA_TERRAIN],
+				    BUTTON_HEIGHT / 2 - width / 2,
+				    BUTTON_HEIGHT / 2 - height / 2);
+	cairo_pattern_set_extend(cairo_get_source(cr),
+				 CAIRO_EXTEND_REPEAT);
+	cairo_paint(cr);
+
+	draw_port_indicator(layout, cr, BUTTON_HEIGHT / 2,
+			    BUTTON_HEIGHT / 2, 11,
+			    GPOINTER_TO_INT(port_type));
+	g_object_unref(layout);
+
+	return TRUE;
+}
+
+/** Draws the unselect button.
+ * @param widget The widget to draw on
+ * @param cr The cairo context
+ * @param user_data Not used
+ * @return TRUE if event is handled
+ */
+static gboolean draw_unselect_cb(G_GNUC_UNUSED GtkWidget * widget,
+				 cairo_t * cr,
+				 G_GNUC_UNUSED gpointer user_data)
+{
+	gdk_cairo_set_source_color(cr, &white);
+	cairo_paint(cr);
+
+	return TRUE;
+}
+
+/** Selects the toolbar button that was clicked.
+ * @param button The GtkButton that was clicked.
+ * @param user_data A ToolbarButtonData to set selected_toolbar_button to.
+ * @return TRUE if event is handled.
+ */
+static gboolean toolbar_click(G_GNUC_UNUSED GtkButton * button,
+			      G_GNUC_UNUSED gpointer user_data)
+{
+	ToolbarButtonData *toolbar_data = user_data;
+	selected_toolbar_button = *toolbar_data;
+	return TRUE;
+}
+
+/** The tooltips for the terrain toolbar buttons.
+ */
+static const gchar *TERRAIN_TOOLBAR_TOOLTIP[] = {
+	N_("Place a hill hex"),
+	N_("Place a field hex"),
+	N_("Place a mountain hex"),
+	N_("Place a pasture hex"),
+	N_("Place a forest hex"),
+	N_("Place a desert hex"),
+	N_("Place a sea hex"),
+	N_("Place a gold hex"),
+	N_("Clear the hex")
+};
+
+/** The tooltips for the chit toolbar buttons.
+ */
+static const gchar *CHIT_TOOLBAR_TOOLTIP[] = {
+	NULL,
+	NULL,
+	N_("Place a 2 chit"),
+	N_("Place a 3 chit"),
+	N_("Place a 4 chit"),
+	N_("Place a 5 chit"),
+	N_("Place a 6 chit"),
+	NULL,
+	N_("Place an 8 chit"),
+	N_("Place a 9 chit"),
+	N_("Place a 10 chit"),
+	N_("Place an 11 chit"),
+	N_("Place a 12 chit")
+};
+
+/** The tooltips for the port toolbar buttons.
+ */
+static const gchar *PORT_TOOLBAR_TOOLTIP[] = {
+	N_("Place a brick 2:1 port"),
+	N_("Place a grain 2:1 port"),
+	N_("Place an ore 2:1 port"),
+	N_("Place a wool 2:1 port"),
+	N_("Place a lumber 2:1 port"),
+	/* Tooltip to remove a port tile in the editor */
+	N_("Remove a port"),
+	N_("Place a 3:1 port")
+};
+
+/** Builds toolbars for selecting terrains, chits, and ports. Places toolbar in 
+ * the table at the top.
+ * @param table The table to place toolbar in.
+ */
+static void build_select_bars(GtkWidget * table)
+{
+	GtkWidget *box;
+	GtkWidget *area;
+	GtkWidget *button;
+	GSList *group;
+	GtkWidget *vsep;
+	gint i;
+	ToolbarButtonData *toolbar_button_data;
+
+	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);	/* upper bar */
+
+	area = gtk_drawing_area_new();
+	gtk_widget_show(area);
+	gtk_widget_set_size_request(area, BUTTON_HEIGHT,
+				    TERRAIN_BUTTON_WIDTH);
+	g_signal_connect(G_OBJECT(area), "draw",
+			 G_CALLBACK(draw_unselect_cb), GINT_TO_POINTER(1));
+
+	button = GTK_WIDGET(gtk_radio_tool_button_new(NULL));
+	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(button), area);
+	gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
+	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(button), _
+				       ("Select the terrain type with the context "
+					"menu on the left click or toggle a node to "
+					"exclude it from being used during the setup "
+					"phase"));
+
+	toolbar_button_data = g_malloc(sizeof(toolbar_button_data));
+	toolbar_button_data->type = NO_TOOLBAR_BUTTON_TYPE;
+
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(toolbar_click), toolbar_button_data);
+
+	/* initialize selected toolbar button to not selected. */
+	selected_toolbar_button.type = NO_TOOLBAR_BUTTON_TYPE;
+	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(button),
+					  TRUE);
+
+	/* Line between unselect and terrains */
+	vsep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
+	gtk_widget_show(vsep);
+	gtk_widget_set_size_request(vsep, 10, 0);
+	gtk_box_pack_start(GTK_BOX(box), vsep, FALSE, TRUE, 0);
+
+	/* terrain toolbar buttons */
+	for (i = 0; i <= LAST_TERRAIN; i++) {
+		area = terrain_icon_new(i);
+		group =
+		    gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON
+						    (button));
+		button = GTK_WIDGET(gtk_radio_tool_button_new(group));
+		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(button),
+						area);
+		gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
+		gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(button),
+					       _(TERRAIN_TOOLBAR_TOOLTIP
+						 [i]));
+
+		toolbar_button_data =
+		    g_malloc(sizeof(toolbar_button_data));
+		toolbar_button_data->type = TERRAIN_TOOLBAR_BUTTON_TYPE;
+		toolbar_button_data->element.terrain = i;
+
+		g_signal_connect(G_OBJECT(button), "clicked",
+				 G_CALLBACK(toolbar_click),
+				 toolbar_button_data);
+
+	}
+
+	gtk_table_attach(GTK_TABLE(table), box, 0, 2, 0, 1,
+			 GTK_FILL, GTK_FILL, 0, 0);
+
+	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);	/* lower bar */
+
+	/* chit toolbar buttons */
+	for (i = 2; i <= 12; i++) {
+		if (i == 7) {
+			continue;
+		}
+		area = gtk_drawing_area_new();
+		gtk_widget_show(area);
+		gtk_widget_set_size_request(area, BUTTON_HEIGHT,
+					    BUTTON_HEIGHT);
+		g_signal_connect(G_OBJECT(area), "draw",
+				 G_CALLBACK(draw_chit_cb),
+				 GINT_TO_POINTER(i));
+
+		group =
+		    gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON
+						    (button));
+		button = GTK_WIDGET(gtk_radio_tool_button_new(group));
+		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(button),
+						area);
+		gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
+		gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(button),
+					       _(CHIT_TOOLBAR_TOOLTIP[i]));
+
+		toolbar_button_data =
+		    g_malloc(sizeof(toolbar_button_data));
+		toolbar_button_data->type = CHIT_TOOLBAR_BUTTON_TYPE;
+		toolbar_button_data->element.chit = i;
+
+		g_signal_connect(G_OBJECT(button), "clicked",
+				 G_CALLBACK(toolbar_click),
+				 toolbar_button_data);
+	}
+
+	/* Line between chit and port */
+	vsep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
+	gtk_widget_show(vsep);
+	gtk_widget_set_size_request(vsep, 10, 0);
+	gtk_box_pack_start(GTK_BOX(box), vsep, FALSE, TRUE, 0);
+
+	/* port toolbar buttons */
+	for (i = 0; i <= ANY_RESOURCE; i++) {
+		area = gtk_drawing_area_new();
+
+		gtk_widget_show(area);
+		gtk_widget_set_size_request(area, BUTTON_HEIGHT,
+					    BUTTON_HEIGHT);
+		g_signal_connect(G_OBJECT(area), "draw",
+				 G_CALLBACK(draw_port_cb),
+				 GINT_TO_POINTER(i));
+
+		group =
+		    gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON
+						    (button));
+		button = GTK_WIDGET(gtk_radio_tool_button_new(group));
+		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(button),
+						area);
+		gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
+		gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(button),
+					       _(PORT_TOOLBAR_TOOLTIP[i]));
+
+		toolbar_button_data =
+		    g_malloc(sizeof(toolbar_button_data));
+		toolbar_button_data->type = PORT_TOOLBAR_BUTTON_TYPE;
+		toolbar_button_data->element.port = i;
+
+		g_signal_connect(G_OBJECT(button), "clicked",
+				 G_CALLBACK(toolbar_click),
+				 toolbar_button_data);
+	}
+
+	gtk_table_attach(GTK_TABLE(table), box, 0, 2, 1, 2,
+			 GTK_FILL, GTK_FILL, 0, 0);
+}
+
 static void build_map_resize(GtkWidget * table, guint col, guint row,
 			     GtkOrientation dir, GtkWidget ** buttons,
 			     GCallback resize_callback)
@@ -236,12 +581,7 @@ static void build_map_resize(GtkWidget * table, guint col, guint row,
 	GtkWidget *box;
 	gint i;
 
-	if (dir == GTK_ORIENTATION_VERTICAL) {
-		box = gtk_vbox_new(FALSE, 0);
-	} else {
-		box = gtk_hbox_new(FALSE, 0);
-	}
-
+	box = gtk_box_new(dir, 0);
 	for (i = 0; i < 4; i++) {
 		buttons[i] =
 		    GTK_WIDGET(gtk_tool_button_new_from_stock(symbols[i]));
@@ -277,8 +617,20 @@ static void scale_map(GuiMap * gmap)
 	gtk_widget_queue_draw(gmap->area);
 }
 
-static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
-				gpointer user_data)
+static gboolean enter_notify_map_cb(GtkWidget * widget,
+				    GdkEventCrossing * event,
+				    G_GNUC_UNUSED gpointer user_data)
+{
+	if (event->mode != GDK_CROSSING_GTK_UNGRAB) {
+		/* Do not grab the focus when a ScrollableTextView
+		 * is focussed on another tab page */
+		gtk_widget_grab_focus(widget);
+	}
+	return FALSE;
+}
+
+static gint button_press_map_cb(GtkWidget * area,
+				GdkEventButton * event, gpointer user_data)
 {
 	GuiMap *gmap = user_data;
 	GtkWidget *menu;
@@ -286,11 +638,12 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 	gboolean port_ok;
 	gint num_ports;
 	gint i;
+	guint idx;
 
 	if (gtk_widget_get_window(area) == NULL || gmap->map == NULL)
 		return FALSE;
 
-	current_hex = guimap_find_hex(gmap, event->x, event->y);
+	current_hex = guimap_get_current_hex(gmap);
 	if (current_hex == NULL)
 		return TRUE;
 
@@ -301,22 +654,77 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 		gint distance_node;
 		gint distance_hex;
 
-		current_node = guimap_find_node(gmap, event->x, event->y);
+		switch (selected_toolbar_button.type) {
+		case TERRAIN_TOOLBAR_BUTTON_TYPE:
+			change_terrain(current_hex,
+				       selected_toolbar_button.
+				       element.terrain);
+			return TRUE;
+		case CHIT_TOOLBAR_BUTTON_TYPE:
+			if (terrain_has_chit(current_hex->terrain)) {
+				current_hex->roll =
+				    selected_toolbar_button.element.chit;
+				guimap_draw_hex(gmap, current_hex);
+			}
+			return TRUE;
+		case PORT_TOOLBAR_BUTTON_TYPE:
+			if (current_hex->terrain == SEA_TERRAIN) {
+				/* If selected resource is NO_RESOURCE, 
+				 * remove port from hex regardless of 
+				 * nearest edge.
+				 */
+				if (selected_toolbar_button.element.port ==
+				    NO_RESOURCE) {
+					current_hex->resource =
+					    NO_RESOURCE;
+					guimap_draw_hex(gmap, current_hex);
+					return TRUE;
+				}
+
+				/* Find nearest edge. This determines the
+				 * direction of the port. 
+				 */
+				const Edge *edge;
+				edge = guimap_get_current_edge(gmap);
+				gint i;
+				for (i = 0; i < 6; i++)
+					if (current_hex->edges[i] == edge)
+						break;
+
+				/* Verify adjacent hex is a land hex */
+				adjacent =
+				    hex_in_direction(current_hex, i);
+				if (adjacent != NULL
+				    && adjacent->terrain != LAST_TERRAIN
+				    && adjacent->terrain != SEA_TERRAIN) {
+					/* change hex resources and direction */
+					current_hex->facing = i;
+					current_hex->resource =
+					    selected_toolbar_button.
+					    element.port;
+					guimap_draw_hex(gmap, current_hex);
+				}
+			}
+			return TRUE;
+		case NO_TOOLBAR_BUTTON_TYPE:
+			break;
+		}
+
+		current_node = guimap_get_current_node(gmap);
 		element.node = current_node;
 		distance_node =
-		    guimap_distance_cursor(gmap, &element, MAP_NODE,
-					   event->x, event->y);
+		    guimap_distance_cursor(gmap, &element, MAP_NODE);
 		element.hex = current_hex;
 		distance_hex =
-		    guimap_distance_cursor(gmap, &element, MAP_HEX,
-					   event->x, event->y);
+		    guimap_distance_cursor(gmap, &element, MAP_HEX);
 
 		if (distance_node < distance_hex) {
 			current_node->no_setup = !current_node->no_setup;
-			for (i = 0; i < G_N_ELEMENTS(current_node->hexes);
-			     i++) {
+			for (idx = 0;
+			     idx < G_N_ELEMENTS(current_node->hexes);
+			     idx++) {
 				guimap_draw_hex(gmap,
-						current_node->hexes[i]);
+						current_node->hexes[idx]);
 			}
 			return TRUE;
 		} else {
@@ -368,22 +776,22 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 static gint key_press_map_cb(GtkWidget * area, GdkEventKey * event,
 			     gpointer user_data)
 {
-	static gint last_x, last_y;
+	static Hex *last_hex = NULL;
 	static gchar *last_key;
 	GuiMap *gmap = user_data;
-	gint x, y;
 	gboolean plus10;
 
 	if (gtk_widget_get_window(area) == NULL || gmap->map == NULL)
 		return FALSE;
 
-	gtk_widget_get_pointer(area, &x, &y);
+	get_mouse_position(area, &gmap->last_x, &gmap->last_y);
 
-	current_hex = guimap_find_hex(gmap, x, y);
+	last_hex = current_hex;
+	current_hex = guimap_get_current_hex(gmap);
 	if (current_hex == NULL || !terrain_has_chit(current_hex->terrain))
 		return TRUE;
 
-	if (last_x == x && last_y == y && strcmp(last_key, "1") == 0)
+	if (last_hex == current_hex && strcmp(last_key, "1") == 0)
 		plus10 = TRUE;
 	else
 		plus10 = FALSE;
@@ -409,8 +817,6 @@ static gint key_press_map_cb(GtkWidget * area, GdkEventKey * event,
 	else if (plus10 && strcmp(event->string, "2") == 0)
 		current_hex->roll = 12;
 	guimap_draw_hex(gmap, current_hex);
-	last_x = x;
-	last_y = y;
 	g_free(last_key);
 	last_key = g_strdup(event->string);
 	return TRUE;
@@ -418,6 +824,9 @@ static gint key_press_map_cb(GtkWidget * area, GdkEventKey * event,
 
 static void update_resize_buttons(void)
 {
+	gboolean can_insert_left;
+	gboolean can_insert_right;
+
 	gtk_widget_set_sensitive(hresize_buttons[RESIZE_REMOVE_LEFT],
 				 gmap->map->x_size > 1 ||
 				 !(gmap->map->shrink_right ||
@@ -426,12 +835,29 @@ static void update_resize_buttons(void)
 				 gmap->map->x_size > 1 ||
 				 !(gmap->map->shrink_right ||
 				   gmap->map->shrink_left));
+	/* Logic table:
+	 * x ? SIZE shrink  result
+	 *   <        X       T (not max)
+	 *   =        F       F (completely full)
+	 *   =        T       shrink (only on one side)
+	 */
+	if (gmap->map->x_size < MAP_SIZE) {
+		/* Not at maximum size */
+		can_insert_left = TRUE;
+		can_insert_right = TRUE;
+	} else if (!(gmap->map->shrink_left || gmap->map->shrink_right)) {
+		/* Completely full */
+		can_insert_left = FALSE;
+		can_insert_right = FALSE;
+	} else {
+		/* Only room on one side */
+		can_insert_left = gmap->map->shrink_left;
+		can_insert_right = gmap->map->shrink_right;
+	}
 	gtk_widget_set_sensitive(hresize_buttons[RESIZE_INSERT_LEFT],
-				 gmap->map->x_size < MAP_SIZE ||
-				 gmap->map->shrink_left);
+				 can_insert_left);
 	gtk_widget_set_sensitive(hresize_buttons[RESIZE_INSERT_RIGHT],
-				 gmap->map->x_size < MAP_SIZE ||
-				 gmap->map->shrink_right);
+				 can_insert_right);
 
 	gtk_widget_set_sensitive(vresize_buttons[RESIZE_REMOVE_TOP],
 				 gmap->map->y_size > 1 &&
@@ -515,7 +941,7 @@ static GtkWidget *build_map(void)
 			      | GDK_BUTTON_PRESS_MASK
 			      | GDK_KEY_PRESS_MASK);
 	g_signal_connect(G_OBJECT(gmap->area), "enter_notify_event",
-			 G_CALLBACK(gtk_widget_grab_focus), gmap);
+			 G_CALLBACK(enter_notify_map_cb), gmap);
 	g_signal_connect(G_OBJECT(gmap->area), "button_press_event",
 			 G_CALLBACK(button_press_map_cb), gmap);
 	g_signal_connect(G_OBJECT(gmap->area), "key_press_event",
@@ -524,6 +950,7 @@ static GtkWidget *build_map(void)
 			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0,
 			 0);
 
+	build_select_bars(table);
 	build_map_resize(table, 1, 2, GTK_ORIENTATION_VERTICAL,
 			 vresize_buttons, G_CALLBACK(change_height));
 	build_map_resize(table, 0, 3, GTK_ORIENTATION_HORIZONTAL,
@@ -532,6 +959,14 @@ static GtkWidget *build_map(void)
 	return table;
 }
 
+static gboolean update_title(GtkWidget * widget,
+			     G_GNUC_UNUSED GdkEvent event,
+			     G_GNUC_UNUSED gpointer user_data)
+{
+	set_window_title(gtk_entry_get_text(GTK_ENTRY(widget)));
+	return FALSE;
+}
+
 /** Builds the comments tab.
  * @return The comments tab.
  */
@@ -540,8 +975,9 @@ static GtkWidget *build_comments(void)
 	GtkWidget *vbox;
 	GtkWidget *widget;
 
-	vbox = gtk_vbox_new(FALSE, 5);
-	widget = gtk_label_new_with_mnemonic("_Title");
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
+	/* Label */
+	widget = gtk_label_new_with_mnemonic(_("_Title"));
 	gtk_widget_show(widget);
 	gtk_misc_set_alignment(GTK_MISC(widget), 0.0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
@@ -550,8 +986,11 @@ static GtkWidget *build_comments(void)
 	gtk_widget_show(game_title);
 	gtk_box_pack_start(GTK_BOX(vbox), game_title, FALSE, FALSE, 0);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(widget), game_title);
+	g_signal_connect(G_OBJECT(game_title), "focus-out-event",
+			 G_CALLBACK(update_title), NULL);
 
-	widget = gtk_label_new_with_mnemonic("_Description");
+	/* Label */
+	widget = gtk_label_new_with_mnemonic(_("_Description"));
 	gtk_widget_show(widget);
 	gtk_misc_set_alignment(GTK_MISC(widget), 0.0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
@@ -568,7 +1007,8 @@ static GtkWidget *build_comments(void)
 	scrollable_text_view_set_text(SCROLLABLE_TEXT_VIEW
 				      (game_description), "");
 
-	widget = gtk_label_new_with_mnemonic("_Comments");
+	/* Label */
+	widget = gtk_label_new_with_mnemonic(_("_Comments"));
 	gtk_widget_show(widget);
 	gtk_misc_set_alignment(GTK_MISC(widget), 0.0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
@@ -588,38 +1028,7 @@ static gint select_terrain_cb(G_GNUC_UNUSED GtkWidget * menu,
 			      gpointer user_data)
 {
 	Terrain terrain = GPOINTER_TO_INT(user_data);
-	Hex *adjacent;
-	gint i;
-
-	if (terrain == current_hex->terrain)
-		return TRUE;
-
-	current_hex->terrain = terrain;
-	if (terrain_has_chit(terrain)) {
-		if (current_hex->roll == 0)
-			current_hex->roll = 2;
-	} else
-		current_hex->roll = 0;
-
-	if (terrain != SEA_TERRAIN)
-		current_hex->resource = NO_RESOURCE;
-	if (terrain == SEA_TERRAIN || terrain == LAST_TERRAIN) {
-		for (i = 0; i < 6; i++) {
-			adjacent = hex_in_direction(current_hex, i);
-			if (adjacent != NULL
-			    && adjacent->resource != NO_RESOURCE
-			    && adjacent->facing == (i + 3) % 6) {
-				adjacent->resource = NO_RESOURCE;
-				adjacent->facing = 0;
-				guimap_draw_hex(gmap, adjacent);
-			}
-		}
-	}
-	guimap_draw_hex(gmap, current_hex);
-
-	/* XXX Since some edges may have changed, we need to redisplay */
-	guimap_display(gmap);
-
+	change_terrain(current_hex, terrain);
 	return TRUE;
 }
 
@@ -654,7 +1063,7 @@ static GtkWidget *build_terrain_menu(void)
 		gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
 		if (height > width / theme->scaledata[i].aspect) {
 			height = width / theme->scaledata[i].aspect;
-		} else if (width > height * theme->scaledata[i].aspect) {
+		} else {
 			width = height * theme->scaledata[i].aspect;
 		}
 
@@ -775,7 +1184,7 @@ static GtkWidget *build_port_menu(void)
 {
 	gint i;
 	GtkWidget *item;
-	GdkPixmap *pixmap;
+	GdkPixbuf *pixbuf;
 	GtkWidget *image;
 	GtkWidget *menu;
 	MapTheme *theme = theme_get_current();
@@ -792,11 +1201,11 @@ static GtkWidget *build_port_menu(void)
 				 G_CALLBACK(select_port_resource_cb),
 				 GINT_TO_POINTER(i));
 
-		pixmap = theme->port_tiles[i];
-		if (i >= NO_RESOURCE || pixmap == NULL)
+		pixbuf = theme->port_tiles[i];
+		if (i >= NO_RESOURCE || pixbuf == NULL)
 			continue;
 
-		image = gtk_image_new_from_pixmap(pixmap, NULL);
+		image = gtk_image_new_from_pixbuf(pixbuf);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 					      image);
 	}
@@ -841,19 +1250,20 @@ static GtkWidget *build_settings(GtkWindow * main_window)
 
 
 	/* vbox */
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
 	/*outer_hbox */
-	outer_hbox = gtk_hbox_new(TRUE, 0);
+	outer_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(outer_hbox), TRUE);
 	gtk_box_pack_start(GTK_BOX(vbox), outer_hbox, TRUE, TRUE, 0);
 
 	/* inner_hbox */
-	inner_hbox = gtk_hbox_new(FALSE, 5);
+	inner_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
 	gtk_box_pack_start(GTK_BOX(outer_hbox), inner_hbox, TRUE, TRUE, 0);
 
 	/* lvbox */
-	lvbox = gtk_vbox_new(FALSE, 10);
+	lvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
 	gtk_box_pack_start(GTK_BOX(inner_hbox), lvbox, TRUE, TRUE, 0);
 
 	/* fix */
@@ -861,16 +1271,16 @@ static GtkWidget *build_settings(GtkWindow * main_window)
 	gtk_box_pack_start(GTK_BOX(inner_hbox), fix, FALSE, TRUE, 0);
 
 	/* inner_hbox */
-	inner_hbox = gtk_hbox_new(FALSE, 5);
+	inner_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
 	gtk_box_pack_start(GTK_BOX(outer_hbox), inner_hbox, TRUE, TRUE, 0);
 
 	/* vsep */
-	vsep = gtk_vseparator_new();
+	vsep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show(vsep);
 	gtk_box_pack_start(GTK_BOX(inner_hbox), vsep, FALSE, FALSE, 0);
 
 	/* rvbox */
-	rvbox = gtk_vbox_new(FALSE, 10);
+	rvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
 	gtk_box_pack_start(GTK_BOX(inner_hbox), rvbox, TRUE, TRUE, 0);
 
 	/* get elements */
@@ -900,12 +1310,13 @@ static GtkWidget *build_settings(GtkWindow * main_window)
 	return vbox;
 }
 
-static void set_window_title(const gchar * title)
+void set_window_title(const gchar * title)
 {
 	gchar *str;
 	g_free(window_title);
 	if (title == NULL) {
-		title = "Untitled";
+		/* The game in the editor does not have a name yet */
+		title = _("Untitled");
 		window_title = NULL;
 	} else
 		window_title = g_strdup(title);
@@ -929,6 +1340,10 @@ static void apply_params(const GameParams * params)
 	game_rules_set_domestic_trade(game_rules, params->domestic_trade);
 	game_settings_set_players(game_settings, params->num_players);
 	game_rules_set_sevens_rule(game_rules, params->sevens_rule);
+	game_rules_set_use_dice_deck(game_rules, params->use_dice_deck);
+	game_rules_set_num_dice_decks(game_rules, params->num_dice_decks);
+	game_rules_set_num_removed_dice_cards(game_rules,
+					      params->num_removed_dice_cards);
 	game_settings_set_victory_points(game_settings,
 					 params->victory_points);
 	/* check_victory_at_end_of_turn not needed in the editor */
@@ -972,6 +1387,10 @@ static GameParams *get_params(void)
 	params->domestic_trade = game_rules_get_domestic_trade(game_rules);
 	params->num_players = game_settings_get_players(game_settings);
 	params->sevens_rule = game_rules_get_sevens_rule(game_rules);
+	params->use_dice_deck = game_rules_get_use_dice_deck(game_rules);
+	params->num_dice_decks = game_rules_get_num_dice_decks(game_rules);
+	params->num_removed_dice_cards =
+	    game_rules_get_num_removed_dice_cards(game_rules);
 	params->victory_points =
 	    game_settings_get_victory_points(game_settings);
 
@@ -1198,7 +1617,7 @@ static void change_title_menu_cb(void)
 
 	vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 
-	hbox = gtk_hbox_new(FALSE, 5);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 
@@ -1207,11 +1626,13 @@ static void change_title_menu_cb(void)
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
+	/* Update the window_title before is it shown */
+	set_window_title(gtk_entry_get_text(GTK_ENTRY(game_title)));
+
 	entry = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(entry), 60);
 	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-	if (window_title != NULL)
-		gtk_entry_set_text(GTK_ENTRY(entry), window_title);
+	gtk_entry_set_text(GTK_ENTRY(entry), window_title);
 
 	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 
@@ -1321,7 +1742,7 @@ static GtkActionEntry entries[] = {
 	 N_("Save as"), save_as_menu_cb},
 	{"ChangeTitle", NULL,
 	 /* Menu entry */
-	 N_("_Change Title"), "<control>T",
+	 N_("Change _Title"), "<control>T",
 	 /* Tooltip for Change Title menu entry */
 	 N_("Change game title"), change_title_menu_cb},
 	{"CheckVP", GTK_STOCK_APPLY,
@@ -1531,7 +1952,7 @@ int main(int argc, char *argv[])
 	roll_menu = build_roll_menu();
 	port_menu = build_port_menu();
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_add(GTK_CONTAINER(toplevel), vbox);
 
 	menubar = gtk_ui_manager_get_widget(ui_manager, "/MainMenu");
